@@ -1,20 +1,33 @@
 /* Copyright (c) 2010, 2013 Daniel Doubrovkine, Markus Karg, All Rights Reserved
+ *
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * You can freely decide which license you want to apply to 
+ * the project.
  * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna.platform.win32;
 
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.nio.ByteOrder;
+import java.util.ArrayList;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
@@ -31,7 +44,7 @@ import com.sun.jna.ptr.PointerByReference;
 
 /**
  * Kernel32 utility API.
- * 
+ *
  * @author dblock[at]dblock.org
  * @author markus[at]headcrashing[dot]eu
  * @author Andreas "PAX" L&uuml;ck, onkelpax-git[at]yahoo.de
@@ -40,7 +53,7 @@ public abstract class Kernel32Util implements WinDef {
 
     /**
      * Get current computer NetBIOS name.
-     * 
+     *
      * @return Netbios name.
      */
     public static String getComputerName() {
@@ -52,33 +65,149 @@ public abstract class Kernel32Util implements WinDef {
         return Native.toString(buffer);
     }
 
+    /**
+     * Invokes {@link Kernel32#LocalFree(Pointer)} and checks if it succeeded.
+     *
+     * @param ptr The {@link Pointer} to the memory to be released - ignored if NULL
+     * @throws Win32Exception if non-{@code ERROR_SUCCESS} code reported
+     */
+    public static void freeLocalMemory(Pointer ptr) {
+        Pointer res = Kernel32.INSTANCE.LocalFree(ptr);
+        if (res != null) {
+            throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+        }
+    }
+
+    /**
+     * Invokes {@link Kernel32#GlobalFree(Pointer)} and checks if it succeeded.
+     *
+     * @param ptr The {@link Pointer} to the memory to be released - ignored if NULL
+     * @throws Win32Exception if non-{@code ERROR_SUCCESS} code reported
+     */
+    public static void freeGlobalMemory(Pointer ptr) {
+        Pointer res = Kernel32.INSTANCE.GlobalFree(ptr);
+        if (res != null) {
+            throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+        }
+    }
+
+    /**
+     * Closes all referenced handles. If an exception is thrown for
+     * a specific handle, then it is accumulated until all
+     * handles have been closed. If more than one exception occurs,
+     * then it is added as a suppressed exception to the first one.
+     * Once closed all handles, the accumulated exception (if any) is thrown
+     *
+     * @param refs The references to close
+     * @see #closeHandleRef(WinNT.HANDLEByReference)
+     */
+    public static void closeHandleRefs(HANDLEByReference... refs) {
+        Win32Exception err = null;
+        for (HANDLEByReference r : refs) {
+            try {
+                closeHandleRef(r);
+            } catch(Win32Exception e) {
+                if (err == null) {
+                    err = e;
+                } else {
+                    err.addSuppressedReflected(e);
+                }
+            }
+        }
+
+        if (err != null) {
+            throw err;
+        }
+    }
+    /**
+     * Closes the handle in the reference
+     *
+     * @param ref The handle reference - ignored if {@code null}
+     * @see #closeHandle(WinNT.HANDLE)
+     */
+    public static void closeHandleRef(HANDLEByReference ref) {
+        closeHandle((ref == null) ? null : ref.getValue());
+    }
+
+    /**
+     * Invokes {@link #closeHandle(WinNT.HANDLE)} on each handle. If an exception
+     * is thrown for a specific handle, then it is accumulated until all
+     * handles have been closed. If more than one exception occurs, then it
+     * is added as a suppressed exception to the first one. Once closed all
+     * handles, the accumulated exception (if any) is thrown
+     *
+     * @param handles The handles to be closed
+     * @see Throwable#getSuppressed()
+     */
+    public static void closeHandles(HANDLE... handles) {
+        Win32Exception err = null;
+        for (HANDLE h : handles) {
+            try {
+                closeHandle(h);
+            } catch(Win32Exception e) {
+                if (err == null) {
+                    err = e;
+                } else {
+                    err.addSuppressedReflected(e);
+                }
+            }
+        }
+
+        if (err != null) {
+            throw err;
+        }
+    }
+
+    /**
+     * Invokes {@link Kernel32#CloseHandle(WinNT.HANDLE)} and checks the success code.
+     * If not successful, then throws a {@link Win32Exception} with the
+     * {@code GetLastError} value
+     *
+     * @param h The handle to be closed - ignored if {@code null}
+     */
+    public static void closeHandle(HANDLE h) {
+        if (h == null) {
+            return;
+        }
+
+        if (!Kernel32.INSTANCE.CloseHandle(h)) {
+            throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+        }
+    }
+
     /**
      * Format a message from the value obtained from
-     * {@link Kernel32#GetLastError} or {@link Native#getLastError}.
-     * 
-     * @param code
-     *            int
+     * {@link Kernel32#GetLastError()} or {@link Native#getLastError()}.
+     *
+     * @param code The error code
      * @return Formatted message.
      */
     public static String formatMessage(int code) {
         PointerByReference buffer = new PointerByReference();
-        if (0 == Kernel32.INSTANCE.FormatMessage(
+        int nLen = Kernel32.INSTANCE.FormatMessage(
                 WinBase.FORMAT_MESSAGE_ALLOCATE_BUFFER
-                        | WinBase.FORMAT_MESSAGE_FROM_SYSTEM
-                        | WinBase.FORMAT_MESSAGE_IGNORE_INSERTS, null, code, 0, // TODO:
-                                                                                // MAKELANGID(LANG_NEUTRAL,
-                                                                                // SUBLANG_DEFAULT)
-                buffer, 0, null)) {
-            throw new LastErrorException(Kernel32.INSTANCE.GetLastError());
+                | WinBase.FORMAT_MESSAGE_FROM_SYSTEM
+                | WinBase.FORMAT_MESSAGE_IGNORE_INSERTS,
+                null,
+                code,
+                0, // TODO: // MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT)
+                buffer, 0, null);
+        if (nLen == 0) {
+            throw new LastErrorException(Native.getLastError());
+        }
+
+        Pointer ptr = buffer.getValue();
+        try {
+            String s = ptr.getWideString(0);
+            return s.trim();
+        } finally {
+            freeLocalMemory(ptr);
         }
-        String s = buffer.getValue().getWideString(0);
-        Kernel32.INSTANCE.LocalFree(buffer.getValue());
-        return s.trim();
     }
 
     /**
      * Format a message from an HRESULT.
-     * 
+     *
      * @param code
      *            HRESULT
      * @return Formatted message.
@@ -92,13 +221,14 @@ public abstract class Kernel32Util implements WinDef {
      * @param code error code
      * @return formatted message
      */
+    @Deprecated
     public static String formatMessageFromHR(HRESULT code) {
         return formatMessage(code.intValue());
     }
 
     /**
      * Format a system message from an error code.
-     * 
+     *
      * @param code
      *            Error code, typically a result of GetLastError.
      * @return Formatted message.
@@ -107,18 +237,18 @@ public abstract class Kernel32Util implements WinDef {
         return formatMessageFromHR(W32Errors.HRESULT_FROM_WIN32(code));
     }
 
-	/**
-	 * @return Obtains the human-readable error message text from the last error
-	 *         that occurred by invocating {@code Kernel32.GetLastError()}.
-	 */
-	public static String getLastErrorMessage() {
-		return Kernel32Util.formatMessageFromLastErrorCode(Kernel32.INSTANCE
-				.GetLastError());
-	}
+    /**
+     * @return Obtains the human-readable error message text from the last error
+     *         that occurred by invocating {@code Kernel32.GetLastError()}.
+     */
+    public static String getLastErrorMessage() {
+        return Kernel32Util.formatMessageFromLastErrorCode(Kernel32.INSTANCE
+                .GetLastError());
+    }
 
     /**
      * Return the path designated for temporary files.
-     * 
+     *
      * @return Path.
      */
     public static String getTempPath() {
@@ -138,7 +268,7 @@ public abstract class Kernel32Util implements WinDef {
 
     /**
      * Returns valid drives in the system.
-     * 
+     *
      * @return A {@link List} of valid drives.
      */
     public static List<String> getLogicalDriveStrings() {
@@ -159,7 +289,7 @@ public abstract class Kernel32Util implements WinDef {
 
     /**
      * Retrieves file system attributes for a specified file or directory.
-     * 
+     *
      * @param fileName
      *            The name of the file or directory.
      * @return The attributes of the specified file or directory.
@@ -185,6 +315,7 @@ public abstract class Kernel32Util implements WinDef {
         }
 
         HANDLE hFile = null;
+        Win32Exception err = null;
         try {
             hFile = Kernel32.INSTANCE.CreateFile(fileName, WinNT.GENERIC_READ,
                     WinNT.FILE_SHARE_READ, new WinBase.SECURITY_ATTRIBUTES(),
@@ -197,25 +328,36 @@ public abstract class Kernel32Util implements WinDef {
 
             int type = Kernel32.INSTANCE.GetFileType(hFile);
             switch (type) {
-            case WinNT.FILE_TYPE_UNKNOWN:
-                int err = Kernel32.INSTANCE.GetLastError();
-                switch (err) {
-                case WinError.NO_ERROR:
-                    break;
-                default:
-                    throw new Win32Exception(err);
-                }
+                case WinNT.FILE_TYPE_UNKNOWN:
+                    int rc = Kernel32.INSTANCE.GetLastError();
+                    switch (rc) {
+                        case WinError.NO_ERROR:
+                            break;
+                        default:
+                            throw new Win32Exception(rc);
+                    }
                 // fall-thru
 
             default:
                 return type;
             }
+        } catch(Win32Exception e) {
+            err = e;
+            throw err;  // re-throw so finally block executed
         } finally {
-            if (hFile != null) {
-                if (!Kernel32.INSTANCE.CloseHandle(hFile)) {
-                    throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+            try {
+                closeHandle(hFile);
+            } catch(Win32Exception e) {
+                if (err == null) {
+                    err = e;
+                } else {
+                    err.addSuppressedReflected(e);
                 }
             }
+
+            if (err != null) {
+                throw err;
+            }
         }
     }
 
@@ -229,7 +371,7 @@ public abstract class Kernel32Util implements WinDef {
 
     /**
      * Get the value of an environment variable.
-     * 
+     *
      * @param name
      *            Name of the environment variable.
      * @return Value of an environment variable.
@@ -265,7 +407,7 @@ public abstract class Kernel32Util implements WinDef {
         if (lpszEnvironmentBlock == null) {
             throw new LastErrorException(Kernel32.INSTANCE.GetLastError());
         }
-        
+
         try {
             return getEnvironmentVariables(lpszEnvironmentBlock, 0L);
         } finally {
@@ -284,7 +426,7 @@ public abstract class Kernel32Util implements WinDef {
      * <B>Note:</B> if the environment block is {@code null} then {@code null}
      * is returned instead of an empty map since we want to distinguish
      * between the case that the data block is {@code null} and when there are
-     * no environment variables (as unlikely as it may be) 
+     * no environment variables (as unlikely as it may be)
      */
     public static Map<String,String> getEnvironmentVariables(Pointer lpszEnvironmentBlock, long offset) {
         if (lpszEnvironmentBlock == null) {
@@ -305,13 +447,13 @@ public abstract class Kernel32Util implements WinDef {
             if (pos < 0) {
                 throw new IllegalArgumentException("Missing variable value separator in " + nvp);
             }
-            
+
             String  name=nvp.substring(0, pos), value=nvp.substring(pos + 1);
             vars.put(name, value);
 
             curOffset += (len + 1 /* skip the ending '\0' */) * stepFactor;
         }
-        
+
         return vars;
     }
 
@@ -333,7 +475,7 @@ public abstract class Kernel32Util implements WinDef {
         if (dataLen == 0) {
             return "";
         }
-        
+
         int         charsLen=asWideChars ? (dataLen / 2) : dataLen;
         char[]      chars=new char[charsLen];
         long        curOffset=offset, stepSize=asWideChars ? 2L : 1L;
@@ -351,7 +493,7 @@ public abstract class Kernel32Util implements WinDef {
                 chars[index] = (char) (b & 0x00FF);
             }
         }
-        
+
         return new String(chars);
     }
 
@@ -388,10 +530,10 @@ public abstract class Kernel32Util implements WinDef {
      *      the assumption is that the environment variable <U>name</U> (at
      *      least) is ASCII.
      *      </LI>
-     *      
+     *
      *      <LI>
      *      Otherwise (i.e., zero charset indicator), it is assumed to be
-     *      a {@code wchar_t} 
+     *      a {@code wchar_t}
      *      </LI>
      * </UL>
      * <B>Note:</B> the code takes into account the {@link ByteOrder} even though
@@ -413,7 +555,7 @@ public abstract class Kernel32Util implements WinDef {
             return isWideCharEnvironmentStringBlock(b0);
         }
     }
-    
+
     private static boolean isWideCharEnvironmentStringBlock(byte charsetIndicator) {
         // assume wchar_t for environment variables represents ASCII letters
         if (charsetIndicator != 0) {
@@ -426,7 +568,7 @@ public abstract class Kernel32Util implements WinDef {
     /**
      * Retrieves an integer associated with a key in the specified section of an
      * initialization file.
-     * 
+     *
      * @param appName
      *            The name of the section in the initialization file.
      * @param keyName
@@ -451,7 +593,7 @@ public abstract class Kernel32Util implements WinDef {
 
     /**
      * Retrieves a string from the specified section in an initialization file.
-     * 
+     *
      * @param lpAppName
      *            The name of the section containing the key name. If this
      *            parameter is {@code null}, the
@@ -516,7 +658,7 @@ public abstract class Kernel32Util implements WinDef {
     /**
      * Convenience method to get the processor information. Takes care of
      * auto-growing the array.
-     * 
+     *
      * @return the array of processor information.
      */
     public static final WinNT.SYSTEM_LOGICAL_PROCESSOR_INFORMATION[] getLogicalProcessorInformation() {
@@ -543,7 +685,7 @@ public abstract class Kernel32Util implements WinDef {
         return (WinNT.SYSTEM_LOGICAL_PROCESSOR_INFORMATION[]) firstInformation
                 .toArray(new WinNT.SYSTEM_LOGICAL_PROCESSOR_INFORMATION[returnedStructCount]);
     }
-    
+
     /**
      * Retrieves all the keys and values for the specified section of an initialization file.
      *
@@ -639,7 +781,7 @@ public abstract class Kernel32Util implements WinDef {
             if (hr != WinError.ERROR_MORE_DATA) {
                 throw new Win32Exception(hr);
             }
-            
+
             int required = lpcchReturnLength.getValue();
             lpszVolumePathNames = new char[required];
             // this time we MUST succeed
@@ -647,7 +789,7 @@ public abstract class Kernel32Util implements WinDef {
                 throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
             }
         }
-        
+
         int bufSize = lpcchReturnLength.getValue();
         return Native.toStringList(lpszVolumePathNames, 0, bufSize);
     }
@@ -660,7 +802,7 @@ public abstract class Kernel32Util implements WinDef {
      * Parses and returns the pure GUID value of a volume name obtained
      * from {@link Kernel32#FindFirstVolume(char[], int)} or
      * {@link Kernel32#FindNextVolume} calls
-     * 
+     *
      * @param volumeGUIDPath
      *              The volume GUID path as returned by one of the above mentioned calls
      * @return The pure GUID value after stripping the &quot;\\?\&quot; prefix and
@@ -675,7 +817,281 @@ public abstract class Kernel32Util implements WinDef {
          || (!volumeGUIDPath.endsWith(VOLUME_GUID_PATH_SUFFIX))) {
             throw new IllegalArgumentException("Bad volume GUID path format: " + volumeGUIDPath);
         }
-        
+
         return volumeGUIDPath.substring(VOLUME_GUID_PATH_PREFIX.length(), volumeGUIDPath.length() - VOLUME_GUID_PATH_SUFFIX.length());
     }
+
+    /**
+     *
+     * This function retrieves the full path of the executable file of a given process.
+     *
+     * @param hProcess
+     *          Handle for the running process
+     * @param dwFlags
+     *          0 - The name should use the Win32 path format.
+     *          1(WinNT.PROCESS_NAME_NATIVE) - The name should use the native system path format.
+     *
+     * @return the full path of the process's executable file of null if failed. To get extended error information,
+     *         call GetLastError.
+     */
+    public static final String QueryFullProcessImageName(HANDLE hProcess, int dwFlags) {
+        char[] path = new char[WinDef.MAX_PATH];
+        IntByReference lpdwSize = new IntByReference(path.length);
+        if (Kernel32.INSTANCE.QueryFullProcessImageName(hProcess, 0, path, lpdwSize))
+            return new String(path).substring(0, lpdwSize.getValue());
+        throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+    }
+
+    /**
+     * Gets the specified resource out of the specified executable file
+     *
+     * @param path
+     *            The path to the executable file
+     * @param type
+     *            The type of the resource (either a type name or type ID is
+     *            allowed)
+     * @param name
+     *            The name or ID of the resource
+     * @return The resource bytes, or null if no such resource exists.
+     * @throws IllegalStateException if the call to LockResource fails
+     */
+    public static byte[] getResource(String path, String type, String name) {
+        HMODULE target = Kernel32.INSTANCE.LoadLibraryEx(path, null, Kernel32.LOAD_LIBRARY_AS_DATAFILE);
+
+        if (target == null) {
+            throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+        }
+
+        Win32Exception err = null;
+        Pointer start = null;
+        int length = 0;
+        byte[] results = null;
+        try {
+            Pointer t = null;
+            try {
+                t = new Pointer(Long.parseLong(type));
+            } catch (NumberFormatException e) {
+                t = new Memory(Native.WCHAR_SIZE * (type.length() + 1));
+                t.setWideString(0, type);
+            }
+
+            Pointer n = null;
+            try {
+                n = new Pointer(Long.parseLong(name));
+            } catch (NumberFormatException e) {
+                n = new Memory(Native.WCHAR_SIZE * (name.length() + 1));
+                n.setWideString(0, name);
+            }
+
+            HRSRC hrsrc = Kernel32.INSTANCE.FindResource(target, n, t);
+            if (hrsrc == null) {
+                throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+            }
+
+            // according to MSDN, on 32 bit Windows or newer, calling FreeResource() is not necessary - and in fact does nothing but return false.
+            HANDLE loaded = Kernel32.INSTANCE.LoadResource(target, hrsrc);
+            if (loaded == null) {
+                throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+            }
+
+            length = Kernel32.INSTANCE.SizeofResource(target, hrsrc);
+            if (length == 0) {
+                throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+            }
+
+            // MSDN: It is not necessary to unlock resources because the system automatically deletes them when the process that created them terminates.
+            // MSDN does not say that LockResource sets GetLastError
+            start = Kernel32.INSTANCE.LockResource(loaded);
+            if (start == null) {
+                throw new IllegalStateException("LockResource returned null.");
+            }
+            // have to capture it into a byte array before you free the library, otherwise bad things happen.
+            results = start.getByteArray(0, length);
+        } catch (Win32Exception we) {
+            err = we;
+        } finally {
+            // from what I can tell on MSDN, the only thing that needs cleanup on this is the HMODULE from LoadLibrary
+            if (target != null) {
+                if (!Kernel32.INSTANCE.FreeLibrary(target)) {
+                    Win32Exception we = new Win32Exception(Kernel32.INSTANCE.GetLastError());
+                    if (err != null) {
+                        we.addSuppressedReflected(err);
+                    }
+                    throw we;
+                }
+            }
+        }
+
+        if (err != null) {
+            throw err;
+        }
+
+        return results;
+    }
+
+    /**
+     * Gets a list of all resources from the specified executable file
+     *
+     * @param path
+     *            The path to the executable file
+     * @return A map of resource type name/ID => resources.<br>
+     *         A map key + a single list item + the path to the executable can
+     *         be handed off to getResource() to actually get the resource.
+     */
+    public static Map<String, List<String>> getResourceNames(String path) {
+        HMODULE target = Kernel32.INSTANCE.LoadLibraryEx(path, null, Kernel32.LOAD_LIBRARY_AS_DATAFILE);
+
+        if (target == null) {
+            throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+        }
+
+        final List<String> types = new ArrayList<String>();
+        final Map<String, List<String>> result = new LinkedHashMap<String, List<String>>();
+
+        WinBase.EnumResTypeProc ertp = new WinBase.EnumResTypeProc() {
+
+            @Override
+            public boolean invoke(HMODULE module, Pointer type, Pointer lParam) {
+                // simulate IS_INTRESOURCE macro defined in WinUser.h
+                // basically that means that if "type" is less than or equal to 65,535
+                // it assumes it's an ID.
+                // otherwise it assumes it's a pointer to a string
+                if (Pointer.nativeValue(type) <= 65535) {
+                    types.add(Pointer.nativeValue(type) + "");
+                } else {
+                    types.add(type.getWideString(0));
+                }
+                return true;
+            }
+        };
+
+        WinBase.EnumResNameProc ernp = new WinBase.EnumResNameProc() {
+
+            @Override
+            public boolean invoke(HMODULE module, Pointer type, Pointer name, Pointer lParam) {
+                String typeName = "";
+
+                if (Pointer.nativeValue(type) <= 65535) {
+                    typeName = Pointer.nativeValue(type) + "";
+                } else {
+                    typeName = type.getWideString(0);
+                }
+
+                if (Pointer.nativeValue(name) < 65535) {
+                    result.get(typeName).add(Pointer.nativeValue(name) + "");
+                } else {
+                    result.get(typeName).add(name.getWideString(0));
+                }
+
+                return true;
+            }
+        };
+
+
+        Win32Exception err = null;
+        try {
+            if (!Kernel32.INSTANCE.EnumResourceTypes(target, ertp, null)) {
+                throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+            }
+
+            for (final String typeName : types) {
+                result.put(typeName, new ArrayList<String>());
+
+                // simulate MAKEINTRESOURCE macro in WinUser.h
+                // basically, if the value passed in can be parsed as a number then convert it into one and run with that.
+                // otherwise, assume it's a string and construct a pointer to said string.
+                Pointer pointer = null;
+                try {
+                    pointer = new Pointer(Long.parseLong(typeName));
+                } catch (NumberFormatException e) {
+                    pointer = new Memory(Native.WCHAR_SIZE * (typeName.length() + 1));
+                    pointer.setWideString(0, typeName);
+                }
+
+                boolean callResult = Kernel32.INSTANCE.EnumResourceNames(target, pointer, ernp, null);
+
+                if (!callResult) {
+                    throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+                }
+            }
+        } catch (Win32Exception e) {
+            err = e;
+        } finally {
+            // from what I can tell on MSDN, the only thing that needs cleanup
+            // on this is the HMODULE from LoadLibrary
+            if (target != null) {
+                if (!Kernel32.INSTANCE.FreeLibrary(target)) {
+                    Win32Exception we = new Win32Exception(Kernel32.INSTANCE.GetLastError());
+                    if (err != null) {
+                        we.addSuppressedReflected(err);
+                    }
+                    throw we;
+                }
+            }
+        }
+
+        if (err != null) {
+            throw err;
+        }
+        return result;
+    }
+
+    /**
+     * Returns all the executable modules for a given process ID.<br>
+     *
+     * @param processID
+     *            The process ID to get executable modules for
+     * @return All the modules in the process.
+     */
+    public static List<Tlhelp32.MODULEENTRY32W> getModules(int processID) {
+        HANDLE snapshot = Kernel32.INSTANCE.CreateToolhelp32Snapshot(Tlhelp32.TH32CS_SNAPMODULE, new DWORD(processID));
+        if (snapshot == null) {
+            throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+        }
+
+        Win32Exception we = null;
+        try {
+            Tlhelp32.MODULEENTRY32W first = new Tlhelp32.MODULEENTRY32W();
+
+            if (!Kernel32.INSTANCE.Module32FirstW(snapshot, first)) {
+                throw new Win32Exception(Kernel32.INSTANCE.GetLastError());
+            }
+
+            List<Tlhelp32.MODULEENTRY32W> modules = new ArrayList<Tlhelp32.MODULEENTRY32W>();
+            modules.add(first);
+
+            Tlhelp32.MODULEENTRY32W next = new Tlhelp32.MODULEENTRY32W();
+            while (Kernel32.INSTANCE.Module32NextW(snapshot, next)) {
+                modules.add(next);
+                next = new Tlhelp32.MODULEENTRY32W();
+            }
+
+            int lastError = Kernel32.INSTANCE.GetLastError();
+            // if we got a false from Module32Next,
+            // check to see if it returned false because we're genuinely done
+            // or if something went wrong.
+            if (lastError != W32Errors.ERROR_SUCCESS && lastError != W32Errors.ERROR_NO_MORE_FILES) {
+                throw new Win32Exception(lastError);
+            }
+
+            return modules;
+        } catch (Win32Exception e) {
+            we = e;
+            throw we;   // re-throw so finally block is executed
+        } finally {
+            try {
+                closeHandle(snapshot);
+            } catch(Win32Exception e) {
+                if (we == null) {
+                    we = e;
+                } else {
+                    we.addSuppressedReflected(e);
+                }
+            }
+
+            if (we != null) {
+                throw we;
+            }
+        }
+    }
 }
