@@ -1,19 +1,31 @@
 /* Copyright (c) 2010 Daniel Doubrovkine, All Rights Reserved
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
  * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * You can freely decide which license you want to apply to 
+ * the project.
+ * 
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna.platform.win32;
 
 import com.sun.jna.Native;
 import com.sun.jna.Pointer;
+import com.sun.jna.Structure;
 import com.sun.jna.platform.win32.Sspi.CredHandle;
 import com.sun.jna.platform.win32.Sspi.CtxtHandle;
 import com.sun.jna.platform.win32.Sspi.PSecPkgInfo;
@@ -30,7 +42,7 @@ import com.sun.jna.win32.W32APIOptions;
  * @author dblock[at]dblock.org
  */
 public interface Secur32 extends StdCallLibrary {
-    Secur32 INSTANCE = (Secur32) Native.loadLibrary("Secur32", Secur32.class, W32APIOptions.UNICODE_OPTIONS);
+    Secur32 INSTANCE = Native.loadLibrary("Secur32", Secur32.class, W32APIOptions.DEFAULT_OPTIONS);
 	
     /**
      * Specifies a format for a directory service object name.
@@ -57,7 +69,7 @@ public interface Secur32 extends StdCallLibrary {
      * @param len On input, the size of the buffer, on output the number of characters copied into the buffer, not including the terminating null character.
      * @return True if the function succeeds. False otherwise.
      */
-    public boolean GetUserNameEx(int nameFormat, char[] lpNameBuffer, IntByReference len);
+    boolean GetUserNameEx(int nameFormat, char[] lpNameBuffer, IntByReference len);
 
     /**
      * The AcquireCredentialsHandle function acquires a handle to preexisting credentials 
@@ -94,7 +106,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the function returns one of the SEC_I_ success codes.
      *  If the function fails, the function returns one of the SEC_E_ error codes.
      */
-    public int AcquireCredentialsHandle(String pszPrincipal, String pszPackage,
+    int AcquireCredentialsHandle(String pszPrincipal, String pszPackage,
                                         int fCredentialUse, LUID pvLogonID,
                                         Pointer pAuthData, Pointer pGetKeyFn, // TODO: SEC_GET_KEY_FN
                                         Pointer pvGetKeyArgument, CredHandle phCredential, 
@@ -161,7 +173,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the function returns one of the SEC_I_ success codes.
      *  If the function fails, the function returns one of the SEC_E_ error codes.
      */
-    public int InitializeSecurityContext(CredHandle phCredential, CtxtHandle phContext,
+    int InitializeSecurityContext(CredHandle phCredential, CtxtHandle phContext,
                                          String pszTargetName, int fContextReq, int Reserved1,
                                          int TargetDataRep, SecBufferDesc pInput, int Reserved2,
                                          CtxtHandle phNewContext, SecBufferDesc pOutput, IntByReference pfContextAttr,
@@ -176,7 +188,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the return value is SEC_E_OK.
      *  If the function fails, the return value is SEC_E_INVALID_HANDLE;
      */
-    public int DeleteSecurityContext(CtxtHandle phContext);
+    int DeleteSecurityContext(CtxtHandle phContext);
 	
     /**
      * The FreeCredentialsHandle function notifies the security system that the 
@@ -191,7 +203,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the return value is SEC_E_OK.
      *  If the function fails, the return value is SEC_E_INVALID_HANDLE;
      */
-    public int FreeCredentialsHandle(CredHandle phCredential);
+    int FreeCredentialsHandle(CredHandle phCredential);
 	
     /**
      * The AcceptSecurityContext function enables the server component of a transport 
@@ -235,7 +247,7 @@ public interface Secur32 extends StdCallLibrary {
      * @return
      *  This function returns one of SEC_* values.
      */
-    public int AcceptSecurityContext(CredHandle phCredential, CtxtHandle phContext,
+    int AcceptSecurityContext(CredHandle phCredential, CtxtHandle phContext,
                                      SecBufferDesc pInput, int fContextReq, int TargetDataRep,
                                      CtxtHandle phNewContext, SecBufferDesc pOutput, IntByReference pfContextAttr,
                                      TimeStamp ptsTimeStamp);
@@ -253,8 +265,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the function returns SEC_E_OK.
      *  If the function fails, it returns a nonzero error code.
      */
-    public int EnumerateSecurityPackages(IntByReference pcPackages, 
-                                         PSecPkgInfo ppPackageInfo);
+    int EnumerateSecurityPackages(IntByReference pcPackages,  PSecPkgInfo ppPackageInfo);
 	
     /**
      * The FreeContextBuffer function enables callers of security package functions to free a memory 
@@ -266,7 +277,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the function returns SEC_E_OK.
      *  If the function fails, it returns a nonzero error code.
      */
-    public int FreeContextBuffer(Pointer buffer);
+    int FreeContextBuffer(Pointer buffer);
 	
     /**
      * The QuerySecurityContextToken function obtains the access token for a client security context
@@ -280,8 +291,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function fails, it returns a nonzero error code. One possible error code return is 
      *  SEC_E_INVALID_HANDLE.
      */
-    public int QuerySecurityContextToken(CtxtHandle phContext, 
-                                         HANDLEByReference phToken);
+    int QuerySecurityContextToken(CtxtHandle phContext, HANDLEByReference phToken);
 	
     /**
      * The ImpersonateSecurityContext function allows a server to impersonate a client by using 
@@ -296,7 +306,7 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function fails, it returns a SEC_E_INVALID_HANDLE, SEC_E_NO_IMPERSONATION or 
      *  SEC_E_UNSUPPORTED_FUNCTION error code.
      */
-    public int ImpersonateSecurityContext(CtxtHandle phContext);
+    int ImpersonateSecurityContext(CtxtHandle phContext);
 	
     /**
      * Allows a security package to discontinue the impersonation of the caller and restore its 
@@ -309,5 +319,25 @@ public interface Secur32 extends StdCallLibrary {
      *  If the function succeeds, the return value is SEC_E_OK.
      *  If the function fails, the return value can be either SEC_E_INVALID_HANDLE or SEC_E_UNSUPPORTED_FUNCTION.
      */
-    public int RevertSecurityContext(CtxtHandle phContext);
+    int RevertSecurityContext(CtxtHandle phContext);
+	
+    /**
+     * Enables a transport application to query a security package for certain
+     * attributes of a security context.
+     * 
+     * @param phContext
+     *  A handle to the security context to be queried.
+     * @param ulAttribute
+     *  Specifies the attribute of the context to be returned. This
+     *  parameter can be one of the SECPKG_ATTR_* values defined in
+     *  {@link Sspi}.
+     * @param pBuffer
+     *  A pointer to a structure that receives the attributes. The
+     *  type of structure pointed to depends on the value specified in
+     *  the ulAttribute parameter.
+     * @return
+     *  If the function succeeds, the return value is SEC_E_OK.
+     *  If the function fails, the return value is a nonzero error code.
+     */
+    int QueryContextAttributes(CtxtHandle phContext, int ulAttribute, Structure pBuffer);
 }
