@@ -1,14 +1,25 @@
 /* Copyright (c) 2007-2013 Timothy Wall, All Rights Reserved
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * <p/>
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
+ * 
+ * You can freely decide which license you want to apply to 
+ * the project.
+ * 
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 
 /* Native library implementation to support JUnit tests. */
@@ -855,12 +866,14 @@ fillDoubleBuffer(double *buf, int len, double value) {
   return len;
 }
 
+#include "ffi.h"
+
 EXPORT int32_t
-addInt32VarArgs(const char *fmt, ...) {
+addVarArgs(const char *fmt, ...) {
   va_list ap;
   int32_t sum = 0;
   va_start(ap, fmt);
-  
+
   while (*fmt) {
     switch (*fmt++) {
     case 'd':
@@ -872,6 +885,10 @@ addInt32VarArgs(const char *fmt, ...) {
     case 'c':
       sum += (int) va_arg(ap, int);
       break;
+    case 'f': // float (promoted to ‘double’ when passed through ‘...’)
+    case 'g': // double
+      sum += (int) va_arg(ap, double);
+      break;
     default:
       break;
     }
@@ -911,6 +928,16 @@ returnStringVarArgs(const char *fmt, ...) {
   return cp;
 }
 
+EXPORT char *
+returnStringVarArgs2(const char *fmt, ...) {
+  char* cp;
+  va_list ap;
+  va_start(ap, fmt);
+  cp = va_arg(ap, char *);
+  va_end(ap);
+  return cp;
+}
+
 #if defined(_WIN32) && !defined(_WIN64) && !defined(_WIN32_WCE)
 ///////////////////////////////////////////////////////////////////////
 // stdcall tests
@@ -999,6 +1026,11 @@ Java_com_sun_jna_PerformanceTest_00024JNILibrary_getpid(JNIEnv *UNUSED(env), jcl
 #endif
 }
 
+EXPORT jclass
+returnClass(JNIEnv *env, jobject arg) {
+  return (*env)->GetObjectClass(env, arg);
+}
+
 #ifdef __cplusplus
 }
 #endif
