@@ -1,27 +1,36 @@
 /* Copyright (c) 2009 Timothy Wall, All Rights Reserved
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * <p/>
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
+ * 
+ * You can freely decide which license you want to apply to 
+ * the project.
+ * 
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna;
 
 import junit.framework.*;
-import com.sun.jna.*;
-import com.sun.jna.ptr.PointerByReference;
-import java.lang.ref.*;
 import java.lang.reflect.Method;
 import java.io.File;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -36,7 +45,7 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
     static class MathLibrary {
 
         public static native double cos(double x);
-        
+
         static {
             Native.register(Platform.MATH_LIBRARY_NAME);
         }
@@ -48,6 +57,8 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
 
     static class CLibrary {
         public static class size_t extends IntegerType {
+            private static final long serialVersionUID = 1L;
+
             public size_t() {
                 super(Native.POINTER_SIZE);
             }
@@ -64,7 +75,7 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
         public static native int strlen(String s1);
         public static native int strlen(Pointer p);
         public static native int strlen(byte[] b);
-        
+
         static {
             Native.register(Platform.C_LIBRARY_NAME);
         }
@@ -87,7 +98,9 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
     }
 
     static class TestLibrary implements TestInterface {
+        @Override
         public native int callInt32CallbackRepeatedly(Int32Callback cb, int arg1, int arg2, int count);
+        @Override
         public native NativeLong callLongCallbackRepeatedly(NativeLongCallback cb, NativeLong arg1, NativeLong arg2, int count);
         static {
             Native.register("testlib");
@@ -98,6 +111,7 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
         public TestLoader() throws MalformedURLException {
             this(null);
         }
+
         public TestLoader(ClassLoader parent) throws MalformedURLException {
             super(Platform.isWindowsCE()
                   ? new URL[] {
@@ -108,12 +122,13 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
                       new File(BUILDDIR + "/test-classes").toURI().toURL(),
                   }, new CloverLoader(parent));
         }
-        protected Class findClass(String name) throws ClassNotFoundException {
+        @Override
+        protected Class<?> findClass(String name) throws ClassNotFoundException {
             String boot = System.getProperty("jna.boot.library.path");
             if (boot != null) {
                 System.setProperty("jna.boot.library.path", "");
             }
-            Class cls = super.findClass(name);
+            Class<?> cls = super.findClass(name);
             if (boot != null) {
                 System.setProperty("jna.boot.library.path", boot);
             }
@@ -130,7 +145,7 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
                     Native.registered(MathLibrary.class));
     }
 
-    private Class returnCallingClass() {
+    private Class<?> returnCallingClass() {
         return Native.getCallingClass();
     }
 
@@ -142,15 +157,15 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
     public void testFindNativeClass() {
         class UnregisterLibrary {
             class Inner {
-                public Class findDirectMappedClass() {
+                public Class<?> findDirectMappedClass() {
                     return findDirectMappedClassInner();
                 }
-                public Class findDirectMappedClassInner() {
+                public Class<?> findDirectMappedClassInner() {
                     return Native.findDirectMappedClass(Native.getCallingClass());
                 };
             }
             public native double cos(double x);
-            public Class findDirectMappedClass() {
+            public Class<?> findDirectMappedClass() {
                 return new Inner().findDirectMappedClass();
             };
         }
@@ -161,20 +176,21 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
     public static class DirectMapping {
         public static class DirectStructure extends Structure {
             public int field;
-            protected List getFieldOrder() {
-                return Arrays.asList(new String[] { "field" });
+            @Override
+            protected List<String> getFieldOrder() {
+                return Arrays.asList("field");
             }
         }
         public static interface DirectCallback extends Callback {
             void invoke();
         }
-        public DirectMapping(Map options) {
+        public DirectMapping(Map<String, ?> options) {
             Native.register(getClass(), NativeLibrary.getInstance("testlib", options));
         }
     }
 
     public void testGetOptionsForDirectMappingWithMemberInitializer() {
-        Class[] classes = {
+        Class<?>[] classes = {
             DirectMapping.class,
             DirectMapping.DirectStructure.class,
             DirectMapping.DirectCallback.class,
@@ -182,20 +198,20 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
         final TypeMapper mapper = new DefaultTypeMapper();
         final int alignment = Structure.ALIGN_NONE;
         final String encoding = System.getProperty("file.encoding");
-        Map options = new HashMap();
+        Map<String, Object> options = new HashMap<String, Object>();
         options.put(Library.OPTION_TYPE_MAPPER, mapper);
         options.put(Library.OPTION_STRUCTURE_ALIGNMENT, alignment);
         options.put(Library.OPTION_STRING_ENCODING, encoding);
         DirectMapping lib = new DirectMapping(options);
-        for (int i=0;i < classes.length;i++) {
-            assertEquals("Wrong type mapper for direct mapping " + classes[i],
-                         mapper, Native.getTypeMapper(classes[i]));
-            assertEquals("Wrong alignment for direct mapping " + classes[i],
-                         alignment, Native.getStructureAlignment(classes[i]));
-            assertEquals("Wrong encoding for direct mapping " + classes[i],
-                         encoding, Native.getStringEncoding(classes[i]));
-            Object last = Native.getLibraryOptions(classes[i]);;
-            assertSame("Options not cached", last, Native.getLibraryOptions(classes[i]));
+        for (Class<?> cls : classes) {
+            assertEquals("Wrong type mapper for direct mapping " + cls,
+                         mapper, Native.getTypeMapper(cls));
+            assertEquals("Wrong alignment for direct mapping " + cls,
+                         alignment, Native.getStructureAlignment(cls));
+            assertEquals("Wrong encoding for direct mapping " + cls,
+                         encoding, Native.getStringEncoding(cls));
+            Object last = Native.getLibraryOptions(cls);
+            assertSame("Options not cached", last, Native.getLibraryOptions(cls));
         }
     }
 
@@ -203,7 +219,9 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
         final static TypeMapper TEST_MAPPER = new DefaultTypeMapper();
         final static int TEST_ALIGNMENT = Structure.ALIGN_DEFAULT;
         final static String TEST_ENCODING = System.getProperty("file.encoding");
-        final static Map TEST_OPTIONS = new HashMap() {
+        final static Map<String, Object> TEST_OPTIONS = new HashMap<String, Object>() {
+            private static final long serialVersionUID = 1L;    // we're not serializing it
+
             {
                 put(Library.OPTION_TYPE_MAPPER, TEST_MAPPER);
                 put(Library.OPTION_STRUCTURE_ALIGNMENT, TEST_ALIGNMENT);
@@ -215,8 +233,9 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
         }
         public static class DirectStructure extends Structure {
             public int field;
-            protected List getFieldOrder() {
-                return Arrays.asList(new String[] { "field" });
+            @Override
+            protected List<String> getFieldOrder() {
+                return Arrays.asList("field");
             }
         }
         public static interface DirectCallback extends Callback {
@@ -225,20 +244,20 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
     }
 
     public void testGetOptionsForDirectMappingWithStaticInitializer() {
-        Class[] classes = {
+        Class<?>[] classes = {
             DirectMappingStatic.class,
             DirectMappingStatic.DirectStructure.class,
             DirectMappingStatic.DirectCallback.class,
         };
-        for (int i=0;i < classes.length;i++) {
-            assertEquals("Wrong type mapper for direct mapping " + classes[i],
-                         DirectMappingStatic.TEST_MAPPER, Native.getTypeMapper(classes[i]));
-            assertEquals("Wrong alignment for direct mapping " + classes[i],
-                         DirectMappingStatic.TEST_ALIGNMENT, Native.getStructureAlignment(classes[i]));
-            assertEquals("Wrong encoding for direct mapping " + classes[i],
-                         DirectMappingStatic.TEST_ENCODING, Native.getStringEncoding(classes[i]));
-            Object last = Native.getLibraryOptions(classes[i]);;
-            assertSame("Options not cached", last, Native.getLibraryOptions(classes[i]));
+        for (Class<?> cls : classes) {
+            assertEquals("Wrong type mapper for direct mapping " + cls,
+                         DirectMappingStatic.TEST_MAPPER, Native.getTypeMapper(cls));
+            assertEquals("Wrong alignment for direct mapping " + cls,
+                         DirectMappingStatic.TEST_ALIGNMENT, Native.getStructureAlignment(cls));
+            assertEquals("Wrong encoding for direct mapping " + cls,
+                         DirectMappingStatic.TEST_ENCODING, Native.getStringEncoding(cls));
+            Object last = Native.getLibraryOptions(cls);
+            assertSame("Options not cached", last, Native.getLibraryOptions(cls));
         }
     }
 
@@ -249,6 +268,7 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
 
     public void testDirectMappingFunctionMapper() {
         FunctionMapper MAPPER = new FunctionMapper() {
+            @Override
             public String getFunctionName(NativeLibrary lib, Method method) {
                 String name = method.getName();
                 if (name.startsWith("_prefixed_")) {
@@ -257,23 +277,68 @@ public class DirectTest extends TestCase implements Paths, GCWaits {
                 return name;
             }
         };
-        Map options = new HashMap();
-        options.put(Library.OPTION_FUNCTION_MAPPER, MAPPER);
+
         try {
             Native.register(RemappedCLibrary.class,
-                            NativeLibrary.getInstance(Platform.C_LIBRARY_NAME, options));
+                    NativeLibrary.getInstance(Platform.C_LIBRARY_NAME, Collections.singletonMap(Library.OPTION_FUNCTION_MAPPER, MAPPER)));
             final String VALUE = getName();
             int len;
 
             len = RemappedCLibrary.$$YJP$$strlen(VALUE);
-            assertEquals(VALUE.length(), len);
+            assertEquals("Mismatched YJP strlen value", VALUE.length(), len);
 
             len = RemappedCLibrary._prefixed_strlen(VALUE);
-            assertEquals(VALUE.length(), len);
-        }
-        catch(Exception e) {
+            assertEquals("Mismatched prefixed strlen value", VALUE.length(), len);
+        } catch(Exception e) {
             fail("Native method was not properly mapped: " + e);
         }
     }
+
+    public static class PointerNativeMapped implements NativeMapped {
+        String nativeMethodName;
+        @Override
+        public PointerNativeMapped fromNative(Object nativeValue, FromNativeContext context) {
+            nativeMethodName = ((MethodResultContext)context).getMethod().getName();
+            return this;
+        }
+        @Override
+        public Object toNative() {
+            return null;
+        }
+        @Override
+        public Class<?> nativeType() {
+            return Pointer.class;
+        }
+    }
+    public static class PointerTypeMapped {
+        String nativeMethodName;
+    }
+    public static class FromNativeTests {
+        static native PointerNativeMapped returnPointerArgument(PointerNativeMapped arg);
+        static native PointerTypeMapped returnPointerArgument(PointerTypeMapped arg);
+    }
+    public void testDirectMappingFromNative() {
+        DefaultTypeMapper mapper = new DefaultTypeMapper();
+        mapper.addTypeConverter(PointerTypeMapped.class, new TypeConverter() {
+            @Override
+            public PointerTypeMapped fromNative(Object nativeValue, FromNativeContext context) {
+                PointerTypeMapped ret = new PointerTypeMapped();
+                ret.nativeMethodName = ((MethodResultContext)context).getMethod().getName();
+                return ret;
+            }
+            @Override
+            public Object toNative(Object value, ToNativeContext context) {
+                return null;
+            }
+            @Override
+            public Class<?> nativeType() {
+                return Pointer.class;
+            }
+        });
+        NativeLibrary lib = NativeLibrary.getInstance("testlib", Collections.singletonMap(Library.OPTION_TYPE_MAPPER, mapper));
+        Native.register(FromNativeTests.class, lib);
+        assertEquals("Failed to access MethodResultContext", "returnPointerArgument", FromNativeTests.returnPointerArgument(new PointerNativeMapped()).nativeMethodName);
+        assertEquals("Failed to access MethodResultContext", "returnPointerArgument", FromNativeTests.returnPointerArgument(new PointerTypeMapped()).nativeMethodName);
+    }
 }
 
